<?php
/**
 * WPAchievements Importer Tool
 *
 * @package     GamiPress\Admin\Tools\WPAchievements\Importer
 * @since       1.0.0
 */
// Exit if accessed directly
if( !defined( 'ABSPATH' ) ) exit;

// TODO: WPAchievements adds settings to automatically award points for specific actions, so they should be turned into points awards
// TODO: Also this settings are integration specific, anyway, is easy move them manually

/**
 * Register WPAchievements Importer Tool meta boxes
 *
 * @since  1.0.0
 *
 * @param array $meta_boxes
 *
 * @return array
 */
function gamipress_wpa_importer_tool_meta_boxes( $meta_boxes ) {

    $prefix = 'gamipress_wpa_importer_';

    // Setup achievement types options
    $achievements_types = gamipress_get_achievement_types();
    $achievements_types_options = array();

    foreach( $achievements_types as $achievements_type_slug => $achievements_type ) {
        $achievements_types_options[$achievements_type_slug] = $achievements_type['plural_name'];
    }

    // Setup points types options
    $points_types = gamipress_get_points_types();
    $points_types_options = array();

    foreach( $points_types as $points_type_slug => $points_type ) {
        $points_types_options[$points_type_slug] = $points_type['plural_name'];
    }

    // Setup rank types options
    $ranks_types = gamipress_get_rank_types();
    $ranks_types_options = array();

    foreach( $ranks_types as $ranks_type_slug => $ranks_type ) {
        $ranks_types_options[$ranks_type_slug] = $ranks_type['plural_name'];
    }

    $meta_boxes['wpa-importer'] = array(
        'title' => __( 'WPAchievements Importer', 'gamipress-wpa-importer' ),
        'fields' => apply_filters( 'gamipress_wpa_importer_tool_fields', array(
            $prefix . 'desc' => array(
                'content' => __( 'This tool will migrate all WPAchievements stored data to GamiPress. All the new content will be <strong>appended</strong> to prevent override anything. WPAchievements points will be added to the current user points. ', 'gamipress-wpa-importer' )
                    . '<br>' . __( '<strong>Important!</strong> Please backup your database before starting this process.', 'gamipress-wpa-importer' )
                    . ' <a  href="javascript:void(0);" onClick="jQuery(this).next(\'p\').slideToggle();">' . __( 'Read some important notes', 'gamipress-wpa-importer' ) . '</a>'
                    . '<p style="display: none;">'
                        . sprintf( __( '<strong>About Twitter/Facebook shares:</strong> To supply WPAchievements content sharing, you can check the <a href="%s" target="_blank">GamiPress - Social Share</a> add-on.', 'gamipress-wpa-importer' ), 'https://gamipress.com/add-ons/gamipress-social-share' )
                        . '<br>' . sprintf( __( '<strong>About Achievement Popups:</strong> To supply WPAchievements popups, you can check the <a href="%s" target="_blank">GamiPress - Notifications</a> add-on.', 'gamipress-wpa-importer' ), 'https://gamipress.com/add-ons/gamipress-notifications' )
                        . '<br>' . sprintf( __( '<strong>About specific trigger events:</strong> For specific trigger events (bbPress, BuddyPress, WooCommerce, Gravitiy Forms, etc ) you need to install each specific <a href="%s" target="_blank">GamiPress integration</a>. <strong>Note:</strong> All GamiPress integrations are completely free!', 'gamipress-wpa-importer' ), admin_url( 'admin.php?page=gamipress_add_ons' ) )
                    . '</p>',
                'type' => 'html',
            ),
            $prefix . 'achievements_achievement_type' => array(
                'name' => __( 'WPAchievements Achievements to', 'gamipress-wpa-importer' ),
                'desc' => __( 'Choose the achievement type you want to import the WPAchievements achievements.', 'gamipress-wpa-importer' ),
                'type' => 'select',
                'options' => $achievements_types_options,
            ),
            $prefix . 'quests_achievement_type' => array(
                'name' => __( 'WPAchievements Quests to', 'gamipress-wpa-importer' ),
                'desc' => __( 'Choose the achievement type you want to import the WPAchievements quests.', 'gamipress-wpa-importer' ),
                'type' => 'select',
                'options' => $achievements_types_options,
            ),
            $prefix . 'points_points_type' => array(
                'name' => __( 'WPAchievements Points to', 'gamipress-wpa-importer' ),
                'desc' => __( 'Choose the points type you want to import the WPAchievements points.', 'gamipress-wpa-importer' ),
                'type' => 'select',
                'options' => $points_types_options,
            ),
            $prefix . 'ranks_rank_type' => array(
                'name' => __( 'WPAchievements Ranks to', 'gamipress-wpa-importer' ),
                'desc' => __( 'Choose the rank type you want to import the WPAchievements ranks.', 'gamipress-wpa-importer' ),
                'type' => 'select',
                'options' => $ranks_types_options,
            ),
            $prefix . 'run' => array(
                'label' => __( 'Start Importing Data', 'gamipress-wpa-importer' ),
                'type' => 'button',
                'button' => 'primary'
            ),
        ) )
    );

    return $meta_boxes;

}
add_filter( 'gamipress_tools_import_export_meta_boxes', 'gamipress_wpa_importer_tool_meta_boxes' );

/**
 * AJAX handler for the WPA Importer achievements import action
 *
 * @since 1.0.0
 */
function gamipress_wpa_importer_ajax_import_achievements() {

    // Check user capabilities
    if( ! current_user_can( gamipress_get_manager_capability() ) ) {
        wp_send_json_error( __( 'You are not allowed to perform this action.', 'gamipress' ) );
    }

    global $wpdb;

    ignore_user_abort( true );
    set_time_limit( 0 );

    $points_types = gamipress_get_points_types();

    // First of all, check the points type
    $desired_points_type = $_REQUEST['points_points_type'];

    if( ! isset( $points_types[$desired_points_type] ) ) {
        wp_send_json_error( __( 'You need to choose a valid points type.', 'gamipress' ) );
    }

    // Our prefix
    $prefix = '_gamipress_';

    // We got our globally points type
    $points_type = $desired_points_type;

    // Let's to clean old imports
    $old_wpa_imports = $wpdb->get_results( $wpdb->prepare(
        "SELECT pm.meta_id, pm.post_id FROM {$wpdb->postmeta} AS pm WHERE pm.meta_key = %s",
        '_gamipress_wpa_legacy_id'
    ) );

    foreach( $old_wpa_imports as $old_wpa_import ) {
        // Delete the old created post
        $wpdb->delete(
            $wpdb->posts,
            array( 'ID' => $old_wpa_import->post_id )
        );

        // Delete the old created meta
        $wpdb->delete(
            $wpdb->postmeta,
            array( 'meta_id' => $old_wpa_import->meta_id )
        );
    }

    $achievements_types = gamipress_get_achievement_types();

    // WPAchievements achievements
    $desired_achievement_type = $_REQUEST['achievements_achievement_type'];

    if( isset( $achievements_types[$desired_achievement_type] ) ) {

        $achievement_type = $desired_achievement_type;

        $wpa_achievements = $wpdb->get_results( $wpdb->prepare(
            "SELECT *
             FROM {$wpdb->posts}
             WHERE post_type = %s
              AND ID NOT IN (
                SELECT pm.meta_value FROM {$wpdb->postmeta} AS pm WHERE pm.meta_key = %s
              )
             ORDER BY ID ASC;",
            'wpachievements',
            '_gamipress_wpa_legacy_id'
        ) );

        foreach( $wpa_achievements as $wpa_achievement ) {

            $type = get_post_meta( $wpa_achievement->ID, '_achievement_type', true );                                     // Step trigger type
            //$associated_id = absint( get_post_meta( $wpa_achievement->ID, '_achievement_associated_id', true ) );   // Step achievement post
            $points = absint( get_post_meta( $wpa_achievement->ID, '_achievement_points', true ) );                 // Achievement points
            $recurring = absint( get_post_meta( $wpa_achievement->ID, '_achievement_recurring', true ) );           // Achievement Maximum earnings
            $occurrences = absint( get_post_meta( $wpa_achievement->ID, '_achievement_occurrences', true ) );        // Step times
            $thumbnail_url = get_post_meta( $wpa_achievement->ID, '_achievement_image', true );                     // Achievement thumbnail

            $trigger = gamipress_wpa_importer_convert_to_gamipress_trigger( $type );

            if( $trigger || $type === 'custom_achievement' ) {

                // First add the achievement post
                $post_data = (array) $wpa_achievement;

                unset( $post_data['ID'] );
                $post_data['post_type'] = $achievement_type;

                $new_achievement_id = wp_insert_post( $post_data );

                if( $new_achievement_id ) {

                    // Update achievement post meta data
                    update_post_meta( $new_achievement_id, $prefix . 'points', $points );
                    update_post_meta( $new_achievement_id, $prefix . 'points_type', $points_type );
                    update_post_meta( $new_achievement_id, $prefix . 'earned_by', ( $type === 'custom_achievement' ? 'admin' : 'triggers' ) );
                    update_post_meta( $new_achievement_id, $prefix . 'maximum_earnings', ( $recurring ? '' : 1 ) );

                    $thumbnail_id = gamipress_wpa_importer_get_thumbnail_id( $thumbnail_url );

                    if( $thumbnail_id ) {
                        update_post_meta( $new_achievement_id, '_thumbnail_id', $thumbnail_id );
                    }

                    // To meet old ID
                    update_post_meta( $new_achievement_id, $prefix . 'wpa_legacy_id', $wpa_achievement->ID );

                    if( $type !== 'custom_achievement' ) {
                        // Next add a single step associated to the achievement post
                        $step_id = wp_insert_post( array(
                            'post_type'   => 'step',
                            'post_status' => 'publish',
                            'post_parent'   => $new_achievement_id,
                            'menu_order'    => 0,
                        ) );

                        // Since 1.5.1 the P2P library has been removed
                        if( version_compare( GAMIPRESS_VER, '1.5.1', '<' ) ) {

                            // Create the P2P connection from the requirement to the badge
                            $p2p_id = p2p_create_connection(
                                'step-to-' . $achievement_type,
                                array(
                                    'from' => $step_id,
                                    'to'   => $new_achievement_id,
                                    'meta' => array(
                                        'date' => current_time( 'mysql' )
                                    )
                                )
                            );

                            // Add relevant meta to our P2P connection
                            p2p_add_meta( $p2p_id, 'order', '0' );

                        }

                        // Update step meta data
                        update_post_meta( $step_id, $prefix . 'trigger_type', $trigger );
                        update_post_meta( $step_id, $prefix . 'count', $occurrences );
                        update_post_meta( $step_id, $prefix . 'limit', '1' );
                        update_post_meta( $step_id, $prefix . 'limit_type', 'unlimited' );

                    }
                }
            }
        }

    }

    // Return a success message
    wp_send_json_success( __( 'WPAchievements achievements has been migrated successfully.', 'gamipress' ) );
}
add_action( 'wp_ajax_gamipress_wpa_importer_import_achievements', 'gamipress_wpa_importer_ajax_import_achievements' );

/**
 * AJAX handler for the WPA Importer quests import action
 *
 * @since 1.0.0
 */
function gamipress_wpa_importer_ajax_import_quests() {

    // Check user capabilities
    if( ! current_user_can( gamipress_get_manager_capability() ) ) {
        wp_send_json_error( __( 'You are not allowed to perform this action.', 'gamipress' ) );
    }

    global $wpdb;

    ignore_user_abort( true );
    set_time_limit( 0 );

    $points_types = gamipress_get_points_types();

    // First of all, check the points type
    $desired_points_type = $_REQUEST['points_points_type'];

    if( ! isset( $points_types[$desired_points_type] ) ) {
        wp_send_json_error( __( 'You need to choose a valid points type.', 'gamipress' ) );
    }

    // Our prefix
    $prefix = '_gamipress_';

    // We got our globally points type
    $points_type = $desired_points_type;

    $achievements_types = gamipress_get_achievement_types();

    // WPAchievements achievements
    $desired_achievement_type = $_REQUEST['quests_achievement_type'];

    if( isset( $achievements_types[$desired_achievement_type] ) ) {

        $achievement_type = $desired_achievement_type;

        $wpa_quests = $wpdb->get_results( $wpdb->prepare(
            "SELECT *
             FROM {$wpdb->posts}
             WHERE post_type = %s
              AND ID NOT IN (
                SELECT pm.meta_value FROM {$wpdb->postmeta} AS pm WHERE pm.meta_key = %s
              )
             ORDER BY ID ASC;",
            'wpquests',
            '_gamipress_wpa_legacy_id'
        ) );

        foreach( $wpa_quests as $wpa_quest ) {

            // First add the achievement post
            $post_data = (array) $wpa_quest;

            unset( $post_data['ID'] );
            $post_data['post_type'] = $achievement_type;

            $new_achievement_id = wp_insert_post( $post_data );

            if( $new_achievement_id ) {

                $points = absint( get_post_meta( $wpa_quest->ID, '_quest_points', true ) );                 // Achievement points
                $thumbnail_url = get_post_meta( $wpa_quest->ID, '_quest_image', true );                     // Achievement thumbnail

                // Update achievement post meta data
                update_post_meta( $new_achievement_id, $prefix . 'points', $points );
                update_post_meta( $new_achievement_id, $prefix . 'points_type', $points_type );
                update_post_meta( $new_achievement_id, $prefix . 'earned_by', 'triggers' );

                $thumbnail_id = gamipress_wpa_importer_get_thumbnail_id( $thumbnail_url );

                if( $thumbnail_id ) {
                    update_post_meta( $new_achievement_id, '_thumbnail_id', $thumbnail_id );
                }

                // To meet old ID
                update_post_meta( $new_achievement_id, $prefix . 'wpa_legacy_id', $wpa_quest->ID );

                // Quest details => Achievement Steps
                $quest_details = get_post_meta( $wpa_quest->ID, '_quest_details', true );

                foreach( $quest_details as $quest_item ) {

                    // Setup step data
                    $type = $quest_item['type'];                        // Step trigger type
                    $occurrences = $quest_item['occurrences'];          // Step times
                    $wpa_specific_achievement_id = absint( $quest_item['ach_id'] );  // Step specific achievement ID
                    $achievement_post = $quest_item['associated_id'];   // Step achievement post

                    $trigger = gamipress_wpa_importer_convert_to_gamipress_trigger( $type );

                    if( $trigger ) {

                        // Extra check for specific achievement
                        if( $trigger === 'specific-achievement' ) {

                            // WPAchievements has stored achievement ID, so we need to find by '_gamipress_wpa_legacy_id' meta key
                            $specific_achievement_id = $wpdb->get_results( $wpdb->prepare(
                                "SELECT pm.post_id FROM {$wpdb->postmeta} AS pm WHERE pm.meta_key = %s AND pm.meta_value = %d",
                                '_gamipress_wpa_legacy_id',
                                $wpa_specific_achievement_id
                            ) );

                            // Unable to establish a relationship, so skip
                            if( ! $specific_achievement_id ) {
                                continue;
                            }

                        }

                        // Next add a single step associated to the achievement post
                        $step_id = wp_insert_post( array(
                            'post_type'   => 'step',
                            'post_status' => 'publish',
                            'post_parent'   => $new_achievement_id,
                            'menu_order'    => 0,
                        ) );

                        // Since 1.5.1 the P2P library has been removed
                        if( version_compare( GAMIPRESS_VER, '1.5.1', '<' ) ) {

                            // Create the P2P connection from the requirement to the badge
                            $p2p_id = p2p_create_connection(
                                'step-to-' . $achievement_type,
                                array(
                                    'from' => $step_id,
                                    'to'   => $new_achievement_id,
                                    'meta' => array(
                                        'date' => current_time( 'mysql' )
                                    )
                                )
                            );

                            // Add relevant meta to our P2P connection
                            p2p_add_meta( $p2p_id, 'order', '0' );

                        }

                        // Update step meta data
                        update_post_meta( $step_id, $prefix . 'trigger_type', $trigger );
                        update_post_meta( $step_id, $prefix . 'count', $occurrences );
                        update_post_meta( $step_id, $prefix . 'limit', '1' );
                        update_post_meta( $step_id, $prefix . 'limit_type', 'unlimited' );

                        if( $trigger === 'specific-achievement' && $specific_achievement_id ) {

                            $specific_achievement_type = get_post_type( $specific_achievement_id );

                            // Since 1.5.1 the P2P library has been removed
                            if( version_compare( GAMIPRESS_VER, '1.5.1', '<' ) ) {

                                // Create a connection between achievement and step
                                p2p_create_connection(
                                    $specific_achievement_type . '-to-step',
                                    array(
                                        'from' => $specific_achievement_id,
                                        'to'   => $step_id,
                                        'meta' => array(
                                            'date' => current_time('mysql')
                                        )
                                    )
                                );

                            }

                            update_post_meta( $step_id, $prefix . 'achievement_type', $specific_achievement_type );
                            update_post_meta( $step_id, $prefix . 'achievement_post', $specific_achievement_id );

                        }

                    }

                }

            }
        }

    }

    // Return a success message
    wp_send_json_success( __( 'WPAchievements quests has been migrated successfully.', 'gamipress' ) );
}
add_action( 'wp_ajax_gamipress_wpa_importer_import_quests', 'gamipress_wpa_importer_ajax_import_quests' );

/**
 * AJAX handler for the WPA Importer points import action
 *
 * @since 1.0.0
 */
function gamipress_wpa_importer_ajax_import_points() {

    // Check user capabilities
    if( ! current_user_can( gamipress_get_manager_capability() ) ) {
        wp_send_json_error( __( 'You are not allowed to perform this action.', 'gamipress' ) );
    }

    global $wpdb;

    ignore_user_abort( true );
    set_time_limit( 0 );

    $points_types = gamipress_get_points_types();

    // First of all, check the points type
    $desired_points_type = $_REQUEST['points_points_type'];

    if( ! isset( $points_types[$desired_points_type] ) ) {
        wp_send_json_error( __( 'You need to choose a valid points type.', 'gamipress' ) );
    }

    // We got our globally points type
    $points_type = $desired_points_type;

    // Default points
    $user_meta = '_gamipress_points';

    if( ! empty( $points_type ) ) {
        $user_meta = "_gamipress_{$points_type}_points";
    }

    // Get all stored users
    $users = $wpdb->get_results( "SELECT {$wpdb->users}.ID FROM {$wpdb->users}" );

    foreach( $users as $user ) {

        // Support for WPAchievements integrations
        if(function_exists( WPACHIEVEMENTS_CUBEPOINTS )) {
            $wpa_user_points = (int) cp_getPoints( $user->ID );
        } elseif(function_exists( WPACHIEVEMENTS_MYCRED )){
            $pointType = get_option('wpachievements_mycred_point_type');
            $wpa_user_points = (int) mycred_get_users_cred( $user->ID, $pointType );
        } else{
            $wpa_user_points = (int) get_user_meta( $user->ID, 'achievements_points', true );
        }

        $user_points = absint( get_user_meta( $user->ID, $user_meta, true ) );

        update_user_meta( $user->ID, $user_meta, ( $user_points + $wpa_user_points ) );

    }

    // Return a success message
    wp_send_json_success( __( 'WPAchievements points has been migrated successfully.', 'gamipress' ) );
}
add_action( 'wp_ajax_gamipress_wpa_importer_import_points', 'gamipress_wpa_importer_ajax_import_points' );

/**
 * AJAX handler for the WPA Importer ranks import action
 *
 * @since 1.0.2
 */
function gamipress_wpa_importer_ajax_import_ranks() {

    // Check user capabilities
    if( ! current_user_can( gamipress_get_manager_capability() ) ) {
        wp_send_json_error( __( 'You are not allowed to perform this action.', 'gamipress' ) );
    }

    global $wpdb;

    ignore_user_abort( true );
    set_time_limit( 0 );

    $points_types = gamipress_get_points_types();

    // First of all, check the points type
    $desired_points_type = $_REQUEST['points_points_type'];

    if( ! isset( $points_types[$desired_points_type] ) ) {
        wp_send_json_error( __( 'You need to choose a valid points type.', 'gamipress' ) );
    }

    // Next, check the rank type
    $rank_types = gamipress_get_rank_types();

    $desired_rank_type = $_REQUEST['ranks_rank_type'];

    if( ! isset( $rank_types[$desired_rank_type] ) ) {
        wp_send_json_error( __( 'You need to choose a valid rank type.', 'gamipress' ) );
    }

    // Our prefix
    $prefix = '_gamipress_';

    // We got our globally points type
    $points_type = $desired_points_type;
    $rank_type = $desired_rank_type;

    $wpa_ranks = (array) wpachievements_get_site_option( 'wpachievements_ranks_data' );

    // Reorder WPAchievements Ranks by required points
    ksort($wpa_ranks);

    $priority = 0;

    foreach( $wpa_ranks as $wpa_rank_points => $wpa_rank ) {

        $rank_data = array(
            'post_type' => $rank_type,
            'post_status' => 'publish',
            'menu_order' => $priority,
        );

        if( is_array( $wpa_rank ) ) {
            $rank_data['post_title'] = $wpa_rank[0];
            $thumbnail_id = gamipress_wpa_importer_get_thumbnail_id( $wpa_rank[1] );
        } else {
            $rank_data['post_title'] = $wpa_rank;
            $thumbnail_id = false;
        }

        $new_rank_id = wp_insert_post( $rank_data );

        if( $new_rank_id ) {

            // Set the thumbnail if exists
            if( $thumbnail_id ) {
                update_post_meta( $new_rank_id, '_thumbnail_id', $thumbnail_id );
            }

            // To meet old ID (WPAchievements do not register any rank ID, so let's to use their required points + rank name as ID)
            update_post_meta( $new_rank_id, $prefix . 'wpa_legacy_id', $wpa_rank_points . $rank_data['post_title'] );

            if( $priority !== 0 ) {
                // Next add a single rank requirement associated to the rank post
                $rank_requirement_id = wp_insert_post( array(
                    'post_type'   => 'rank-requirement',
                    'post_status' => 'publish',
                    'post_parent'   => $new_rank_id,
                    'menu_order'    => 0,
                ) );

                // Since 1.5.1 the P2P library has been removed
                if( version_compare( GAMIPRESS_VER, '1.5.1', '<' ) ) {

                    // Create the P2P connection from the requirement to the badge
                    $p2p_id = p2p_create_connection(
                        'rank-requirement-to-' . $rank_type,
                        array(
                            'from' => $rank_requirement_id,
                            'to'   => $new_rank_id,
                            'meta' => array(
                                'date' => current_time( 'mysql' )
                            )
                        )
                    );

                    // Add relevant meta to our P2P connection
                    p2p_add_meta( $p2p_id, 'order', '0' );

                }

                // Update rank requirement meta data
                update_post_meta( $rank_requirement_id, $prefix . 'trigger_type', 'earn-points' );
                update_post_meta( $rank_requirement_id, $prefix . 'points_required', $wpa_rank_points );
                update_post_meta( $rank_requirement_id, $prefix . 'points_type_required', $points_type );
            }

            // Increase priority for next rank
            $priority++;
        }
    }

    // Return a success message
    wp_send_json_success( __( 'WPAchievements ranks has been migrated successfully.', 'gamipress' ) );

}
add_action( 'wp_ajax_gamipress_wpa_importer_import_ranks', 'gamipress_wpa_importer_ajax_import_ranks' );

/**
 * AJAX handler for the WPA Importer earnings import action
 *
 * @since 1.0.0
 */
function gamipress_wpa_importer_ajax_import_earnings() {

    // Check user capabilities
    if( ! current_user_can( gamipress_get_manager_capability() ) ) {
        wp_send_json_error( __( 'You are not allowed to perform this action.', 'gamipress' ) );
    }

    global $wpdb;

    ignore_user_abort( true );
    set_time_limit( 0 );

    $points_types = gamipress_get_points_types();

    // First of all, check the points type
    $desired_points_type = $_REQUEST['points_points_type'];

    if( ! isset( $points_types[$desired_points_type] ) ) {
        wp_send_json_error( __( 'You need to choose a valid points type.', 'gamipress' ) );
    }

    // We got our globally points type
    $points_type = $desired_points_type;

    $achievements_types = gamipress_get_achievement_types();

    // WPAchievements achievements
    $desired_achievements_achievement_type = $_REQUEST['achievements_achievement_type'];

    if( isset( $achievements_types[$desired_achievements_achievement_type] ) ) {
        $achievements_achievement_type = $desired_achievements_achievement_type;
    }

    // WPAchievements quests
    $desired_quests_achievement_type = $_REQUEST['quests_achievement_type'];

    if( isset( $achievements_types[$desired_quests_achievement_type] ) ) {
        $quests_achievement_type = $desired_quests_achievement_type;
    }

    $rank_types = gamipress_get_rank_types();

    $desired_rank_type = $_REQUEST['ranks_rank_type'];

    if( isset( $rank_types[$desired_rank_type] ) ) {
        $rank_type = $desired_rank_type;
    }

    // WPAchievements ranks
    $wpa_ranks = (array) wpachievements_get_site_option( 'wpachievements_ranks_data' );

    // Reorder WPAchievements Ranks by required points
    ksort($wpa_ranks);

    // Get all stored users
    $users = $wpdb->get_results( "SELECT {$wpdb->users}.ID FROM {$wpdb->users}" );

    $ct_table = ct_setup_table( 'gamipress_user_earnings' );

    foreach( $users as $user ) {

        // Achievements earned
        $user_earned_achievements = get_user_meta( $user->ID, 'achievements_gained', true );

        if( is_array( $user_earned_achievements ) ) {

            foreach( $user_earned_achievements as $wpa_achievement_id ) {

                $post_id = $wpdb->get_results( $wpdb->prepare(
                    "SELECT pm.post_id FROM {$wpdb->postmeta} AS pm WHERE pm.meta_key = %s AND pm.meta_value = %d",
                    '_gamipress_wpa_legacy_id',
                    $wpa_achievement_id
                ) );

                if( $post_id ) {

                    $wpa_points = get_post_meta( $wpa_achievement_id, '_achievement_points', true );

                    $earning_data = array(
                        'user_id' => $user->ID,
                        'post_id' => $post_id,
                        'post_type' => get_post_type( $post_id ),
                        'points' => absint( $wpa_points ),
                        'points_type' => $points_type,
                        'date' => date( 'Y-m-d H:i:s' )
                    );

                    $ct_table->db->insert( $earning_data );
                }

            }

        }

        // Quest earned
        $user_earned_quests = get_user_meta( $user->ID, 'quests_gained', true );

        if( is_array( $user_earned_quests ) ) {

            foreach( $user_earned_quests as $wpa_quest_id ) {

                $post_id = $wpdb->get_results( $wpdb->prepare(
                    "SELECT pm.post_id FROM {$wpdb->postmeta} AS pm WHERE pm.meta_key = %s AND pm.meta_value = %d",
                    '_gamipress_wpa_legacy_id',
                    $wpa_quest_id
                ) );

                if( $post_id ) {

                    $wpa_points = get_post_meta( $wpa_quest_id, '_quest_points', true );

                    $earning_data = array(
                        'user_id' => $user->ID,
                        'post_id' => $post_id,
                        'post_type' => get_post_type( $post_id ),
                        'points' => absint( $wpa_points ),
                        'points_type' => $points_type,
                        'date' => date( 'Y-m-d H:i:s' )
                    );

                    $ct_table->db->insert( $earning_data );
                }

            }

        }

        // Rank reached
        $user_points = gamipress_get_user_points( $user->ID, $points_type );

        foreach( $wpa_ranks as $wpa_rank_points => $wpa_rank ) {

            // Skip default rank
            if( $wpa_rank_points === 0 ) {
                continue;
            }

            // user just can reach a rank by points
            if( $user_points >= $wpa_rank_points ) {

                // Setup a custom legacy ID based on rank required points + rank title
                if( is_array( $wpa_rank ) ) {
                    $rank_title = $wpa_rank[0];
                } else {
                    $rank_title = $wpa_rank;
                }

                $post_id = $wpdb->get_results( $wpdb->prepare(
                    "SELECT pm.post_id FROM {$wpdb->postmeta} AS pm WHERE pm.meta_key = %s AND pm.meta_value = %d",
                    '_gamipress_wpa_legacy_id',
                    $wpa_rank_points . $rank_title
                ) );

                // Just reach rank if is with a higher priority
                if( gamipress_get_rank_priority( $post_id ) > gamipress_get_rank_priority( gamipress_get_user_rank_id( $user->ID, $rank_type ) ) ) {
                    gamipress_update_user_rank( $user->ID, $post_id );
                }

            }

        }

    }

    // Return a success message
    wp_send_json_success( __( 'WPAchievements earnings has been migrated successfully.', 'gamipress' ) );
}
add_action( 'wp_ajax_gamipress_wpa_importer_import_earnings', 'gamipress_wpa_importer_ajax_import_earnings' );

/**
 * Turns a WPAcheivements type to a GamiPress trigger
 *
 * @param $type
 *
 * @return string|false
 */
function gamipress_wpa_importer_convert_to_gamipress_trigger( $type ) {

    switch( $type ) {

        // WordPress triggers
        case 'user_register':
        case 'user_login':
            $trigger = 'gamipress_login';
            break;
        case 'post_added':
            $trigger = 'gamipress_publish_post';
            break;
        case 'comment_added':
            $trigger = 'gamipress_new_comment';
            break;
        case 'user_post_view':
            $trigger = 'gamipress_site_visit';
            break;
        case 'user_post_viewed':
            $trigger = 'gamipress_user_post_visit';
            break;
        case 'wpachievements_achievement':
            $trigger = 'specific-achievement';
            break;

        // BuddyPress

        case 'cp_bp_avatar_uploaded':
            $trigger = 'gamipress_bp_upload_avatar';
            break;
        case 'cp_bp_message_sent':
            $trigger = 'gamipress_bp_send_message';
            break;
        case 'cp_bp_new_friend':
        // case 'cp_bp_lost_friend':  // Not supported yet
            $trigger = 'gamipress_bp_friendship_accepted';
            break;
        case 'cp_bp_update':
        // case 'cp_bp_reply': // Not supported yet
        // case 'cp_bp_update_removed': // Not supported yet
            $trigger = 'gamipress_bp_publish_activity';
            break;
        case 'cp_bp_group_create':
        // case 'cp_bp_group_delete': // Not supported yet
            $trigger = 'gamipress_bp_new_group';
            break;
        case 'cp_bp_group_joined':
        // case 'cp_bp_group_left': // Not supported yet
            $trigger = 'gamipress_bp_join_group';
            break;
        case 'cp_bp_group_reply':
            // case 'cp_bp_group_avatar_uploaded': // Not supported yet
            $trigger = 'gamipress_bp_group_publish_activity';
            break;

        // bbPress

        case 'bbp_new_forum':
        // case 'bbp_edit_forum': // Not supported yet
        // case 'bbp_deleted_forum': // Not supported yet
            $trigger = 'gamipress_bbp_new_forum';
            break;
        case 'bbp_new_topic':
        // case 'bbp_closed_topic': // Not supported yet
        // case 'bbp_merged_topic': // Not supported yet
        // case 'bbp_post_split_topic': // Not supported yet
        // case 'bbp_sticked_topic': // Not supported yet
        // case 'bbp_unsticked_topic': // Not supported yet
            $trigger = 'gamipress_bbp_new_topic';
            break;
        case 'bbp_new_reply':
        // case 'bbp_deleted_reply': // Not supported yet
            $trigger = 'gamipress_bbp_new_reply';
            break;

        // Gravity Forms

        case 'gform_sub':
            $trigger = 'gamipress_gf_new_form_submission';
            break;

        // LearnDash
        case 'ld_quiz_pass':
            $trigger = 'gamipress_ld_pass_quiz';
            break;
        case 'ld_quiz_perfect':
            $trigger = 'gamipress_ld_complete_quiz';
            break;
        case 'ld_quiz_fail':
            $trigger = 'gamipress_ld_fail_quiz';
            break;
        case 'ld_lesson_complete':
            $trigger = 'gamipress_ld_complete_lesson';
            break;
        case 'ld_course_complete':
            $trigger = 'gamipress_ld_complete_course';
            break;

        // WooCommerce

        case 'wc_order_complete':
        //case 'wc_user_spends': // Not supported yet
            $trigger = 'gamipress_wc_new_purchase';
            break;

        default:
            $trigger = false;
            break;
    }

    return $trigger;

}

// Retrieves the attachment ID from the file URL
function gamipress_wpa_importer_get_thumbnail_id( $image_url ) {

    global $wpdb;

    $attachment = $wpdb->get_col($wpdb->prepare("SELECT ID FROM $wpdb->posts WHERE guid='%s';", $image_url ));

    return isset($attachment[0]) ? $attachment[0] : false;

}